@@ -1,7 +1,7 @@
 #!/usr/bin/perl --
 
 use strict;
-use HTTPD::WatchLog 0.02;
+use HTTPD::WatchLog 0.03;
 $| = 1;
 
 my $log = new HTTPD::WatchLog;
@@ -13,6 +13,12 @@ $log->addr2host(1);
 # pattern quote on
 $log->quote(1);
 
+# pack multibyte chars on
+$log->pack(1);
+
+# set width
+$log->width( 120 );
+
 # check error request
 $log->highlight( ' 404 ', ' 500 ' );
 
