@@ -1,7 +1,7 @@
 #!/usr/bin/perl --
 
 use strict;
-use HTTPD::WatchLog 0.02;
+use HTTPD::WatchLog 0.03;
 $| = 1;
 
 my $log = new HTTPD::WatchLog;
@@ -11,11 +11,17 @@ $file = $ARGV[0] if $ARGV[0] and -r $ARGV[0];
 $log->file( $file );
 
 # turn on DNS lookup
-#$log->addr2host(1);
+$log->addr2host(1);
+
+# epoch2date
+$log->epoch2date(1);
 
 # pattern quote on
 $log->quote(1);
 
+# set width
+$log->width(120);
+
 # cached req
 $log->highlight( ' TCP_HIT/200 ', ' TCP_REFRESH_HIT/200 ', ' TCP_MEM_HIT/200 ' );
 $log->highlight( ' TCP_REFRESH_HIT/304', ' TCP_IMS_HIT/304' );
@@ -27,12 +33,7 @@ my $sub = sub {
     if $line =~ m|\.mpg|;
 };
 
-my $sub2 = sub {
-  my $line = shift;
-  $line =~ s@^[\d\.]{1,}\s|\t@@;
-};
-
-$log->trigger( $sub, $sub2 );
+$log->trigger( $sub );
 
 # go!
 $log->watch;
